from manga_py.provider import Provider
from .helpers.std import Std


class RawLHCom(Provider, Std):
    _root_uri = None

    def get_chapter_index(self) -> str:
        re = self.re.compile(r'-chapter-(.+)\.html')
        return re.search(self.chapter).group(1)

    def get_main_content(self):
        content = self._storage.get('main_content', None)
        if content is not None:
            return content
        return self.http_get(self._root_uri)

    def get_manga_name(self) -> str:
        url = self.get_url()
        if ~url.find('/read-'):
            title = self.html_fromstring(url, '.navbar-brand.manga-name', 0)
            self._root_uri = self.http().normalize_uri(title.get('href'))
        else:
            self._root_uri = url
            title = self.document_fromstring(self.content, '.manga-info h1', 0)
        return title.text_content().strip(' \t\r\n\0')

    def get_chapters(self):
        return self._elements('#tab-chapper a.chapter')

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '.chapter-content img.chapter-img')

    def get_cover(self) -> str:
        return self._cover_from_content('.info-cover img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = RawLHCom



from .helpers.std import Std
from .readmanga_me import ReadmangaMe


class MintMangaCom(ReadmangaMe, Std):

    def get_manga_name(self):
        return self._get_name(r'\.com/([^/]+)')


main = MintMangaCom



import requests

from .url_normalizer import normalize_uri


class Request:
    __redirect_base_url = ''
    _headers = None
    referer = ''
    proxies = None
    allow_webp = True
    user_agent = '{} {} {} {}'.format(
        'Mozilla/5.0 (Windows NT 10.0; Win64; x64)',
        'AppleWebKit/537.36 (KHTML, like Gecko)',
        'Chrome/60.0.3112.101',
        'Safari/537.36'
    )
    default_lang = 'ru-RU,ru;q=0.8,en-US;q=0.5,en;q=0.3'
    cookies = None
    kwargs = None
    debug = False
    response = None
    _history = None
    allow_send_referer = True

    def __init__(self):
        self.proxies = {}
        self.cookies = {}
        self._history = []

    def __patch_headers(self, headers):
        if isinstance(self._headers, dict):
            for i in self._headers:
                headers[i] = self._headers[i]
        return headers

    def _get_cookies(self, cookies=None):
        return cookies if cookies else self.cookies

    def _prepare_redirect_base_url(self, url):
        if not self.__redirect_base_url:
            self.__redirect_base_url = url

    def _get_kwargs(self):
        kwargs = {}
        if self.kwargs:
            kwargs = self.kwargs
        return kwargs

    def __update_cookies(self, r):
        _ = r.cookies.get_dict()
        for c in _:
            self.cookies[c] = _[c]

    def __redirect_helper(self, r, url, method):
        proxy = None
        location = url
        if r.status_code == 303:
            method = 'get'
        elif r.status_code == 305:
            proxy = {
                'http': r.headers['location'],
                'https': r.headers['location'],
            }
        else:
            location = normalize_uri(r.headers['location'], self.__redirect_base_url)
        return proxy, location, method

    def _requests_helper(
            self, method, url, headers=None, data=None,
            max_redirects=10, **kwargs
    ) -> requests.Response:
        self._prepare_redirect_base_url(url)
        headers = self.__patch_headers(headers)
        args = {
            'url': url,
            'headers': headers,
            'data': data,
        }
        self.__set_defaults(args, kwargs)
        self.__set_defaults(args, self._get_kwargs())
        args.setdefault('allow_redirects', False)
        r = getattr(requests, method)(**args)
        self.__update_cookies(r)
        if r.is_redirect and method != 'head':
            if max_redirects < 1:
                self.debug and print(self._history)
                raise AttributeError('Too many redirects')
            self._history.append(url)
            proxy, location, method = self.__redirect_helper(r, url, method)
            if proxy:
                kwargs['proxies'] = proxy
            return self._requests_helper(
                method=method, url=location, headers=headers,
                data=data, max_redirects=(max_redirects - 1),
                **kwargs
            )
        return r

    @staticmethod
    def __set_defaults(args_orig: dict, args_vars: dict):
        for idx in args_vars:
            args_orig.setdefault(idx, args_vars[idx])

    def requests(
            self, url: str, headers: dict = None, cookies: dict = None,
            data=None, method='get', files=None, timeout=None, **kwargs
    ) -> requests.Response:
        if not isinstance(headers, dict):
            headers = {}
        self._history = []
        cookies = self._get_cookies(cookies)
        headers.setdefault('User-Agent', self.user_agent)
        if self.allow_send_referer and self.referer:
            headers.setdefault('Referer', self.referer)
        headers.setdefault('Accept-Language', self.default_lang)
        if self.allow_webp:
            headers['Accept'] = 'text/html,application/xhtml+xml,application/xml;q=1.0,image/webp,image/apng,*/*;q=1.0'
        kwargs.setdefault('proxies', self.proxies)
        self.response = self._requests_helper(
            method=method, url=url, headers=headers, cookies=cookies,
            data=data, files=files, timeout=timeout,
            **kwargs
        )
        return self.response

    def get(self, url: str, headers: dict = None, cookies: dict = None, **kwargs) -> str:
        response = self.requests(
            url=url,
            headers=headers,
            cookies=cookies,
            method='get',
            **kwargs
        )
        text = response.text
        response.close()
        return text

    def post(self, url: str, headers: dict = None, cookies: dict = None, data: dict = (), files=None, **kwargs) -> str:
        response = self.requests(
            url=url,
            headers=headers,
            cookies=cookies,
            method='post',
            data=data,
            files=files,
            **kwargs
        )
        text = response.text
        response.close()
        return text

    def reset_proxy(self):
        self.proxies = {}

    def set_proxy(self, proxy):
        self.reset_proxy()
        if isinstance(proxy, dict):
            self.proxies['http'] = proxy.get('http', None)
            self.proxies['https'] = proxy.get('https', None)
        elif isinstance(proxy, str):
            self.proxies['http'] = proxy

    def get_base_cookies(self, url: str):
        """
        :param url:
        :return:
        """
        response = self.requests(url=url, method='head')
        response.close()
        return response.cookies



from manga_py.provider import Provider
from .helpers.std import Std


class ReadComicOnlineTo(Provider, Std):
    def get_archive_name(self) -> str:
        chapter = self.re.search(r'id=(\d+)', self.chapter).group(1)
        return self.normal_arc_name([self.chapter_id, chapter])

    def get_chapter_index(self, no_increment=False) -> str:
        return str(self.chapter_id)

    def get_main_content(self):
        return self._get_content(r'{}/Comic/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'/Comic/([^/]+)')

    def get_chapters(self):
        return self._elements('table.listing td > a')

    def prepare_cookies(self):
        self.cf_protect(self.get_url())
        self._storage['cookies']['rco_quality'] = 'hq'

    def get_files(self):
        content = self.http_get(self.chapter + '&readType=1')
        items = self.re.findall(r'lstImages.push\("([^"]+)"\)', content)
        return items

    def get_cover(self):
        return self._cover_from_content('.rightBox .barContent img[width]')


main = ReadComicOnlineTo



from os import chmod
from sys import platform
from zipfile import ZipFile

from requests import get

from manga_py.fs import is_file, dirname, path_join, get_util_home_path


class WebDriver:
    driver_version = '2.40'

    @staticmethod
    def is_win():
        return ~platform.find('win32')

    def download_drivder(self):
        url_prefix = 'https://chromedriver.storage.googleapis.com/'
        url = '/chromedriver_linux64.zip'
        if ~platform.find('darwin'):
            url = '/chromedriver_mac64.zip'
        if self.is_win():
            url = '/chromedriver_win32.zip'

        path = path_join(get_util_home_path(), 'driver.zip')

        with open(path, 'wb') as driver:
            driver.write(get(url_prefix + self.driver_version + url).content)
            driver.close()
        with ZipFile(path) as file:
            file.extractall(dirname(self._driver_path()))

    def _driver_path(self):
        if self.is_win():
            driver = 'chromedriver.exe'
        else:
            driver = 'chromedriver'
        return path_join(get_util_home_path(), driver)

    def get_driver(self):
        from selenium import webdriver  # need, if captcha detected
        driver_path = self._driver_path()
        if not is_file(driver_path):
            self.download_drivder()
        self.is_win() or chmod(driver_path, 0o755)
        driver = webdriver.Chrome(executable_path=driver_path)
        driver.set_window_size(500, 600)
        return driver



from urllib.parse import urlparse

from manga_py.http.url_normalizer import normalize_uri
from manga_py.provider import Provider


class JavZipOrg:
    parser = None
    url = None
    domain = None

    def __init__(self, parser: Provider):
        self.parser = parser
        url = parser.chapter
        if parser.re.search(r'jav-zip\.org', url):
            self.url = url
            _ = urlparse(url)
            self.domain = _.scheme + '://' + _.netloc

    def _parse_id(self):
        return self.parser.re.search('/.p=(\d+)', self.url).group(1)

    def parse_images(self, content):
        images = []
        for i in content.cssselect('img'):
            src = normalize_uri(i.get('src'), self.url)
            images.append(src)
        return images

    def get(self, step):
        url = '{}/wp-admin/admin-ajax.php?post={}&action=get_content&step={}'
        url = url.format(self.domain, self._parse_id, step)
        content = self.parser.json.loads(self.parser.http_get(url))
        content = self.parser.document_fromstring(content['mes'])
        allow_more = True
        if len(content.cssselect('a.view-more')) < 1:
            allow_more = False
        return allow_more, content

    def get_images(self):
        if not self.url:
            return []
        images = []
        step = 0
        allow_more = True
        while allow_more:
            allow_more, content = self.get(step)
            step += 50  # constant
            images += self.parse_images(content)
        return images



from manga_py.provider import Provider
from .helpers.std import Std


class NineAnimeCom(Provider, Std):
    _ch = None

    def get_chapter_index(self) -> str:
        ch_result = self._ch.search(self.chapter)
        if ch_result:
            return ch_result.group(1).replace('_', '-')
        return '000-' + self.chapter_id

    def get_main_content(self):
        return self._get_content('{}/manga/{}.html?waring=1')

    def get_manga_name(self) -> str:
        return self._get_name(r'/manga/(.+)\.html')

    def get_chapters(self):
        return self._elements('.detail-chlist a')

    def get_files(self):
        parser = self.html_fromstring(self.chapter.rstrip('/') + '-0-1.html')
        return self._images_helper(parser, 'img.manga_pic')

    def get_cover(self) -> str:
        return self._cover_from_content('img.detail-cover')

    def book_meta(self) -> dict:
        pass

    def prepare_cookies(self):
        self._ch = self.re.compile(r'/chapter/.*?(?:_((?:\d+)(?:_\d+)?))[^/]*?[/]')


main = NineAnimeCom



from .gomanga_co import GoMangaCo

from .helpers.std import Std


class HentaiCafe(GoMangaCo, Std):
    _name_re = r'\.cafe(?:/manga/read)?/([^/]+)/'
    _content_str = '{}/{}/'
    _chapters_selector = '.content .last .x-btn'  # TODO

    def get_archive_name(self) -> str:
        return 'archive_{:0>2}'.format(self.chapter_id)

    def get_chapter_index(self) -> str:
        return str(self.chapter_id)

    def get_cover(self) -> str:
        return self._cover_from_content('.entry-content img')


main = HentaiCafe



from manga_py.provider import Provider
from .helpers.std import Std


class TruyenChonCom(Provider, Std):
    __subtype = None

    def get_chapter_index(self) -> str:
        re = self.re.compile(r'/chap.*?-(\d+(?:\.\d+)?)')
        return re.search(self.chapter).group(1).replace('.', '-')

    def get_main_content(self):
        truyen = 'truyen'
        if ~self.domain.find('nettruyen.'):
            truyen = 'truyen-tranh'
        return self._get_content('{}/%s/{}' % truyen)

    def get_manga_name(self) -> str:
        groups = self.re.search(r'/(truyen.*?)/([^/]+)', self.get_url())
        self.__subtype = groups.group(1)
        return groups.group(2)

    def get_chapters(self):
        return self._elements('.list-chapter .chapter > a')

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '.page-chapter > img', 'data-original')

    def get_cover(self) -> str:
        return self._cover_from_content('.col-image > img')

    def book_meta(self) -> dict:
        pass


main = TruyenChonCom



from manga_py.provider import Provider
from .helpers.std import Std


class FreeMangaTo(Provider, Std):

    def get_chapter_index(self) -> str:
        re = self.re.compile(r'[Cc]hapter\s(\d+(?:\.\d+)?)')
        chapter = re.search(self.chapter[0]).group(1)
        return chapter.replace('.', '-')

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/(?:manga|chapter)/([^/]+)')

    def get_chapters(self):
        items = self._elements('.readIcon a')
        n = self.http().normalize_uri
        return [(i.text_content(), n(i.get('href'))) for i in items]

    def get_files(self):
        content = self.http_get(self.chapter[1])
        images = self.re.search(r'image:\s*(\[.+\])', content)
        return self.json.loads(images.group(1))

    def get_cover(self) -> str:
        return self._cover_from_content('.tooltips > img')

    def book_meta(self) -> dict:
        pass

    def chapter_for_json(self) -> str:
        return self.chapter[1]


main = FreeMangaTo



from .manganelo_com import MangaNeloCom


class MangaFansUs(MangaNeloCom):
    chapter_re = r'[/-]chap(?:ter)?[_-](\d+(?:\.\d+)?(?:-v\d)?)'

    def get_chapter(self):
        return '%s/0' % self.chapter

    def get_manga_name(self) -> str:
        _ = self.re.search(r'(/(?:read-)?manga/)([^/]+)', self.get_url())
        self._prefix = _.group(1)
        return _.group(2)


main = MangaFansUs



from manga_py.provider import Provider
from .helpers.std import Std


class PlusComicoJp(Provider, Std):

    def get_chapter_index(self) -> str:
        return self.re.search('/manga/\d+/(\d+)', self.chapter).group(1)

    def get_main_content(self):
        content = self._storage.get('main_content', None)
        if content:
            return content
        idx = self.re.search('/manga/(\d+)', self.get_url())
        url = '{}/manga/{}/'.format(self.domain, idx.group(1))
        return self.http_get(url)

    def get_manga_name(self) -> str:
        return self.text_content(self.content, '.stage__body h1')

    def get_chapters(self):
        idx = self.re.search(r'/manga/(\d+)', self.get_url()).group(1)
        data = self.http_post('{}/api/getArticleList.nhn'.format(
            self.domain
        ), data={
            'titleNo': idx
        })
        json = self.json.loads(data)
        items = []
        for i in json.get('result', {}).get('list', {}):
            if i.get('freeFlg', 'N') == 'Y':
                items.append(i.get('articleDetailUrl'))
        return items[::-1]

    def get_files(self):
        content = self.http_get(self.chapter)
        images = [i.get('src') for i in self.document_fromstring(content, '.comic-image > img')]
        images += self.re.findall(r'\'(http://comicimg.comico.jp/onetimecontents/.+)\'', content)
        return images

    def get_cover(self) -> str:
        item = self.document_fromstring(self.content, '.stage div[class^="article-hero"]')
        if item:
            return self.parse_background(item[0])

    def save_file(self, idx=None, callback=None, url=None, in_arc_name=None):
        if in_arc_name is None:
            in_arc_name = '{}_image.jpg'.format(idx)
        super().save_file(idx, callback, url, in_arc_name)

    def book_meta(self) -> dict:
        # todo meta
        pass


main = PlusComicoJp



from sys import stderr

from manga_py.provider import Provider
from .helpers.std import Std


class ComicoJp(Provider, Std):

    def get_chapter_index(self) -> str:
        idx = self.re.search(r'articleNo=(\d+)', self.chapter)
        if idx:
            return '{}-{}'.format(self.chapter_id, idx.group(1))
        return str(self.chapter_id)

    def get_main_content(self):
        title_no = self.re.search(r'\.jp/.+titleNo=(\d+)', self.get_url())
        if title_no:
            content = self.http_post('{}/api/getArticleList.nhn'.format(self.domain), data={
                'titleNo': title_no.group(1)
            })
            try:
                return self.json.loads(content).get('result', {}).get('list', [])
            except TypeError:
                pass
        return []

    def get_manga_name(self):
        content = self.http_get(self.get_url())
        name = self.text_content(content, 'title')
        return name[:name.rfind('|')].strip(' \n\t\r')

    def get_chapters(self):
        # TODO: see i['freeFlg'] Y = true, W = false #19
        items = [i['articleDetailUrl'] for i in self.content if i['freeFlg'] == 'Y']
        self.log('Free chapters count: %d' % len(items), file=stderr)
        return items[::-1]

    def get_files(self):
        items = self.html_fromstring(self.chapter, '.comic-image._comicImage > img.comic-image__image')
        return [i.get('src') for i in items]

    def get_cover(self):
        pass

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ComicoJp



from manga_py.provider import Provider
from .helpers.std import Std


class BoredomSocietyXyz(Provider, Std):
    def get_chapter_index(self) -> str:
        return self.re.search(
            r'/reader/\d+/(\d+(?:\.\d+)?)',
            self.chapter
        ).group(1).replace('.', '-')

    def get_main_content(self):
        idx = self.re.search(
            '/(?:titles/info|reader)/(\d+)',
            self.get_url()
        ).group(1)
        return self.http_get('{}/titles/info/{}'.format(
            self.domain,
            idx
        ))

    def get_manga_name(self) -> str:
        return self.text_content(self.content, 'h2')

    def get_chapters(self):
        return self._elements('a.titlesinfo_chaptertitle')

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        images = self._images_helper(parser, 'img.reader_mangaimage')
        n = self.http().normalize_uri
        return [n(i) for i in images]

    def get_cover(self) -> str:
        return self._cover_from_content('img.titlesinfo_coverimage')

    def book_meta(self) -> dict:
        pass

    def prepare_cookies(self):
        # enable "all-images-on-page"
        self.http_post('{}/module/reader/ajax.php'.format(self.domain), data={
            'readingtype': 'all'
        })


main = BoredomSocietyXyz



from manga_py.provider import Provider
from .helpers.std import Std


class MangaNeloCom(Provider, Std):
    chapter_re = r'[/-]chap(?:ter)?[_-](\d+(?:\.\d+)?)'
    _prefix = '/manga/'

    def get_chapter_index(self) -> str:
        return self.re.search(self.chapter_re, self.get_chapter())\
            .group(1).replace('.', '-')

    def get_main_content(self):
        return self._get_content('{}%s{}' % self._prefix)

    def get_manga_name(self) -> str:
        return self._get_name('/(?:manga|chapter)/([^/]+)')

    def get_chapters(self):
        return self._elements('.chapter-list a')

    def get_files(self):
        parser = self.html_fromstring(self.get_chapter())
        images = self._images_helper(parser, '#vungdoc img')
        if not len(images):
            images = self._images_helper(parser, '.vung_doc img,.vung-doc img')
        return images

    def get_cover(self) -> str:
        return self._cover_from_content('.manga-info-pic img')

    def get_chapter(self):
        return self.chapter


main = MangaNeloCom



from .gomanga_co import GoMangaCo


class JapScanCom(GoMangaCo):
    _name_re = r'\.(?:com|cc|to)/[^/]+/([^/]+)/'
    _content_str = '{}/manga/{}/'
    _chapters_selector = '#chapters_list .chapters_list a'

    def get_archive_name(self) -> str:
        idx = self.chapter_id, self.get_chapter_index()
        return self.normal_arc_name({'vol': idx})

    def get_chapter_index(self) -> str:
        selector = r'\.(?:com|cc|to)/[^/]+/[^/]+/(\d+)/'
        url = self.chapter
        return self.re.search(selector, url).group(1)

    def get_files(self):
        n = self.http().normalize_uri
        parser = self.html_fromstring(self.chapter)

        base_url = self.base_url(parser)
        images = self._images_helper(parser, '#pages option', 'data-img')

        return [n(base_url+i) for i in images]

    def base_url(self, parser):
        base_url = parser.cssselect('#image')[0].get('data-src')
        return self.re.search(r'(.+/)\w+\.\w+', base_url).group(1)


main = JapScanCom



from manga_py.fs import basename
from manga_py.provider import Provider
from .helpers.std import Std


class NudeMoonMe(Provider, Std):
    __content = None
    __url = None

    def get_archive_name(self) -> str:
        return 'archive'

    def get_chapter_index(self) -> str:
        return '0'

    def get_main_content(self):
        return self.__content

    def get_manga_name(self) -> str:
        self.__url = self.get_url()
        if not self.re.search(r'\d+-online--', self.__url):
            _url = self.re.search(r'(.+?/\d+)--(.+\.html)', self.__url)
            self.__url = '{}-online--{}'.format(*_url.groups())
        self.__content = self.http_get(self.__url + '?row')
        name = self.document_fromstring(self.__content, 'meta[property="og:title"]', 0).get('content')
        name = self.re.search(r'(.+?) [#/]', name)
        return name.group(1) if name else basename(self.__url)

    def get_chapters(self):
        return [b'']

    def get_files(self):
        return [i.get('src') for i in self._elements('.square-red center > img')]

    def get_cover(self) -> str:
        pass

    def book_meta(self) -> dict:
        # todo meta
        pass

    def chapter_for_json(self):
        return self.__url


main = NudeMoonMe



from manga_py.provider import Provider
from .helpers.std import Std


class MangaMewCom(Provider, Std):
    _type = 'manga'

    def get_chapter_index(self) -> str:
        re = r'%s/[^/]+/.+?-(\d+(?:-\d+)?)-\d+' % self._type
        return self.re.search(re, self.chapter).group(1)

    def get_main_content(self):
        url = self.get_url()
        if url.find('/' + self._type + '/') == -1:  # not found
            a = self.html_fromstring(url, 'h1.name a', 0)
            url = a.get('href')
        return self.http_get(url)

    def get_manga_name(self) -> str:
        content = self.http_get(self.get_url())
        return self.text_content(content, 'h1.name a,h1.title')

    def get_chapters(self):
        return self._elements('.chapter .item a')[::-1]

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '#content .item > img')

    def get_cover(self) -> str:
        return self._cover_from_content('.images img')

    def book_meta(self) -> dict:
        pass


main = MangaMewCom



from manga_py.provider import Provider
from .helpers.std import Std


class NiAddCom(Provider, Std):

    def get_chapter_index(self) -> str:
        re = self.re.compile(r'/chapter/.*?_(\d+(?:_\d+)?)/')
        return re.search(self.chapter).group(1).replace('_', '-')

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)')

    def get_chapters(self):
        return self._elements('.detail-chlist > a')

    def get_files(self):
        img_selector = 'img.manga_pic'
        url = self.chapter + '-10-{}.html'
        parser = self.html_fromstring(url.format(1))
        pages = len(self._first_select_options(parser, '.sl-page')) + 1
        images = self._images_helper(parser, img_selector)
        for p in range(2, pages):
            parser = self.html_fromstring(url.format(p))
            images += self._images_helper(parser, img_selector)
        return images

    def get_cover(self) -> str:
        return self._cover_from_content('.manga-detailtop img')

    def book_meta(self) -> dict:
        pass


main = NiAddCom



from manga_py.provider import Provider
from .helpers.std import Std


class ReadComicsOnlineRu(Provider, Std):
    def get_chapter_index(self, no_increment=False) -> str:
        chapter = self.re.search(r'/([\w\d_-]+)$', self.chapter).group(1)
        return chapter.replace('.', '-').replace('_', '-')

    def get_main_content(self):
        return self._get_content(r'{}/comic/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'/comic/([^/]+)')

    def get_chapters(self):
        return self._elements('.chapters .chapter-title-rtl a')

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '#all > img', 'data-src')

    def get_cover(self):
        return self._cover_from_content('.boxed .img-responsive')


main = ReadComicsOnlineRu



from manga_py.provider import Provider
from .helpers.std import Std


class TruyenTranhTuanCom(Provider, Std):

    def get_chapter_index(self) -> str:
        idx = self.re.search(r'\.com/[^/]+?-(\d+(?:-\d+)?)', self.chapter)
        return idx.group(1)

    def get_main_content(self):
        content = self.http_get(self.get_url())
        parser = self.document_fromstring(content, '#read-title a.mangaName')
        if parser and len(parser):
            return self.http_get(self.http().normalize_uri(parser[0].get('href')))
        return content

    def get_manga_name(self) -> str:
        url = self.get_url()
        if self.re.search(r'\.com/[^/]+-\d+/', url):
            parser = self.html_fromstring(url, '#read-title .mangaName', 0)
            url = parser.get('href')
        return self.re.search(r'\.com/([^/]+)/', url).group(1)

    def get_chapters(self):
        return self._elements('#manga-chapter .chapter-name a')

    def get_files(self):
        content = self.http_get(self.chapter)
        items = self.re.search(r'slides_page_url_path\s*=\s*(\[.+\])[;,]?', content)
        if items:
            n = self.http().normalize_uri
            items = self.json.loads(items.group(1))
            return [n(i) for i in items]
        return []

    def get_cover(self) -> str:
        return self._cover_from_content('.manga-cover img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = TruyenTranhTuanCom



from manga_py.provider import Provider
from .helpers.std import Std


class MangaDexCom(Provider, Std):
    _links_on_page = 100
    _home_url = ''

    def get_archive_name(self) -> str:
        vol = self.chapter['vol']
        if len(vol) == 0:
            vol = '0'
        return self.normal_arc_name({
            'vol': vol,
            'ch': [*self.chapter['ch'].split('.'), self.chapter['lng']],
        })

    def get_chapter_index(self) -> str:
        fmt = '{}-{}'
        if len(self.chapter['lng']) > 0:
            fmt += '-{}'
        return fmt.format(
            self.chapter['ch'].replace('.', '-'),
            self.chapter['vol'],
            self.chapter['lng'],
        )

    def get_main_content(self):
        url = self.get_url()
        if url.find('/title/') < 0:
            url = self.html_fromstring(url, 'a.manga-link', 0)
            url = self.http().normalize_uri(url.get('href'))
        self._home_url = self.re.search(r'(.+/title/\d+/[^/])', url).group(1)
        return self.http_get(self._home_url)

    def get_manga_name(self) -> str:
        url = self.get_url()
        if ~url.find('/title/'):
            name = self.html_fromstring(url, '.card-header', 0).text_content()
        else:
            name = self.html_fromstring(url, '.manga-link', 0).get('title')
        return name.strip()

    def _all_langs(self, items):
        languages = []
        for i in items:
            languages.append(i['flag'] + '\t--- ' + i['lng'])
        return list(set(languages))

    def _filter_langs(self, chapters, lng):
        if len(lng) < 1:
            return chapters
        lng = lng.split(' ')
        result = []
        for i in chapters:
            if i['flag'] == '' or i['flag'] in lng:
                result.append(i)
        return result

    def get_chapters(self):
        parser = self.document_fromstring(self.content)
        # https://mangadex.org/manga/153/detective-conan
        pages = parser.cssselect('.pagination li.paging a')
        items = self._get_chapters_links(parser)
        if pages:
            pages = self.re.search(r'.+/(\d+)', pages[0].get('href')).group(1)
            for i in range(2, int(pages) + 1):
                _parser = self.html_fromstring('{}/chapters/{}/'.format(
                    self._home_url, i
                ))
                items += self._get_chapters_links(_parser)
        chapters = self._parse_chapters(items)
        lng = self.quest(
            [],
            'Available languages:\n{}\n\nPlease, select your lang (empty for all, space for delimiter lang):'.format(
                '\n'.join(self._all_langs(chapters))
            ))
        return self._filter_langs(chapters, lng)

    def _get_chapters_links(self, parser):
        return parser.cssselect('div.chapter-row[data-chapter]')

    def get_files(self):
        idx = self.re.search(r'/chapter/(\d+)', self.chapter['link']).group(1)
        try:
            data = self.json.loads(self.http_get('{}/api/chapter/{}'.format(
                self.domain, idx
            )))
            n = self.http().normalize_uri
            items = []
            for item in data.get('page_array', []):
                items.append('{}{}/{}'.format(
                    n(data.get('server', '/data/')), data.get('hash'), item
                ))
            return items
        except Exception as e:
            return []

    def get_cover(self) -> str:
        return self._cover_from_content('.card-body .rounded')

    def prepare_cookies(self):
        self._storage['cookies']['mangadex_h_toggle'] = '1'

    def _parse_chapters(self, items):
        n = self.http().normalize_uri
        result = []
        for tr in items:
            ch = tr.cssselect('a[href*="/chapter/"]')[0]
            lng = tr.cssselect('span.flag')
            _ch = {
                'ch': tr.get('data-chapter'),
                'vol': tr.get('data-volume'),
                'link': n(ch.get('href')),
            }
            if lng:
                _ch['lng'] = lng[0].attrib['title']
                _ch['flag'] = lng[0].attrib['class'].replace('rounded flag flag-', '')
            else:
                _ch['lng'] = ''
                _ch['flag'] = ''
            result.append(_ch)
        return result

    def book_meta(self) -> dict:
        # todo meta
        pass

    def chapter_for_json(self):
        return self.chapter['link']


main = MangaDexCom



from random import randrange

from manga_py.provider import Provider
from .helpers.std import Std


class ManhuaTaiCom(Provider, Std):
    servers = [
        'http://mhpic.mh51.com',
        'http://mhpic.manhualang.com',
        'http://mhpic.jumanhua.com',
        'http://mhpic.yyhao.com',
    ]

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index()
        return self.normal_arc_name({'vol': [
            self.chapter_id, idx
        ]})

    def get_chapter_index(self) -> str:
        ch = self.chapter
        return self.re.search(r'/([^/]+)\.html', ch).group(1)

    def get_main_content(self):
        return self._get_content('{}/{}/')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.com/([^/]+)')

    def get_chapters(self):
        topics = self._elements('[id^=topic]')
        items = []
        for i in topics[::-1]:
            items += i.cssselect('a')
        return items

    @staticmethod
    def _decode_img_path(page_id, img_path):
        result = ''
        pid = int(page_id) % 10
        for i in img_path:
            result += chr(ord(i) - pid)
        return result

    def get_server(self):
        idx = randrange(0, len(self.servers))
        return self.servers[idx]

    def get_files(self):
        content = self.http_get(self.chapter)
        pageid = self.re.search(r'pageid:\s*(\d+)', content).group(1)
        imgpath = self.re.search(r'imgpath:\s*[\'"](.+?)[\'"]', content).group(1)
        startimg = self.re.search(r'startimg:\s*(\d+)', content).group(1)
        totalimg = self.re.search(r'totalimg:\s*(\d+)', content).group(1)
        comic_size = self.re.search(r'comic_size:\s*[\'"](.+?)[\'"]', content).group(1)

        imgpath = self._decode_img_path(pageid, imgpath)

        items = []
        for i in range(int(startimg), int(totalimg) + 1):
            items.append('{}/comic/{}{}.jpg{}'.format(
                self.get_server(),
                imgpath,
                i,
                comic_size
            ))
        return items

    def get_cover(self) -> str:
        return self._cover_from_content('.comic-cover > img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ManhuaTaiCom



from .gomanga_co import GoMangaCo


class ReadYagamiMe(GoMangaCo):
    _name_re = '/read/[^/]+/([^/]+)/'
    _content_str = '{}/series/{}/'
    _chapter_re = r'/rea\w+/[^/]+/(?:[^/]+/)?(\d+/\d+(?:/\d+)?)'

    def get_main_content(self):
        return self.http_get(self.get_url())

    def get_manga_name(self) -> str:
        e = self._elements('h1.title')[0]
        return e.text.strip()

    def get_files(self):
        try:
            return super().get_files()
        except AttributeError:
            # web manga
            parser = self.document_fromstring(self._go_chapter_content)
            return self._images_helper(parser, '.web_pictures > img')

    def _get_json_selector(self, content):
        return r'pages\s*=\s*(\[.+?\])'

    def prepare_cookies(self):
        response = self.http().requests(method='post', data={'adult': 'true'}, url=self.get_url())
        cookies = response.cookies.items()
        for i in cookies:
            self._storage['cookies'][i[0]] = i[1]


main = ReadYagamiMe



from manga_py.provider import Provider
from .helpers.std import Std


class MerakiScansCom(Provider, Std):
    _content_url = '{}/manga/{}/'

    def get_chapter_index(self) -> str:
        re = self.re.compile(r'/manga/[^/]+/(\d+(?:\.\d+)?)')
        idx = re.search(self.chapter).group(1)
        return idx.replace('.', '-')

    def _home_url(self):
        return self._content_url.format(self.domain, self.manga_name)

    def get_main_content(self):
        return self.http_get(self._home_url())

    def get_manga_name(self) -> str:
        return self._get_name('com/manga/([^/]+)')

    def get_chapters(self):
        selector = '.clickable-chapter'
        items = self._elements(selector)
        return [i.get('data-href') for i in items]

    def get_files(self):
        content = self.http_get(self.chapter)
        slug = self.re.search(r'manga_slug\s*=\s*[\'"](.+)[\'"]', content).group(1)
        chapter = self.re.search(r'viewschapter\s*=\s*[\'"](.+)[\'"]', content).group(1)
        images = self.re.search(r'images\s*=\s*(\[.+\])', content).group(1).replace('\'', '"')
        images = self.json.loads(images)

        # SRC RULE: "/manga/" + manga_slug + "/" + currentChapter + "/" + images[pageNum - 1];

        return ['{}/manga/{}/{}/{}'.format(self.domain, slug, chapter, i) for i in images]

    def get_cover(self) -> str:
        return self._cover_from_content('#cover_img')

    def book_meta(self) -> dict:
        # todo meta
        pass

    def prepare_cookies(self):
        self.http().cookies['reading_type'] = 'long'


main = MerakiScansCom



from manga_py.provider import Provider
from .helpers.std import Std


class ManhwaHentaiCom(Provider, Std):

    def get_chapter_index(self) -> str:
        return self.re.search(r'/manhwa/[^/]+/[\w-]+-(\d+(?:-\d+)?)', self.chapter).group(1)

    def get_main_content(self):
        return self._get_content('{}/manhwa/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'./manhwa/([^/]+)')

    def get_chapters(self):
        return self._elements('.version-chap > li > a')

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '.reading-content .wp-manga-chapter-img')

    def get_cover(self) -> str:
        return self._cover_from_content('.summary_image img.img-responsive')

    def book_meta(self) -> dict:
        pass


main = ManhwaHentaiCom



from manga_py.meta import __downloader_uri__
from manga_py.provider import Provider


class TapasIo:
    provider = None

    def __init__(self, provider: Provider):
        self.provider = provider

    def _content(self, content):
        type = content.get('type', None)
        if type == 'DEFAULT':
            return self._type_default(content)

    def _error(self, content):
        self.provider.log('\r\nERROR!\r\nCode: {}\r\nType: {}\r\nPlease, send url to developer ({})'.format(
            content['code'],
            content['type'],
            __downloader_uri__
        ))

    def _type_default(self, content):
        items = self.provider.document_fromstring(content.get('data', {}).get('html', '<html></html>'), '.art-image')
        return [i.get('src') for i in items]

    def chapter_url(self):
        return '{}/episode/view/{}'.format(
            self.provider.domain,
            self.provider.chapter['id']
        )

    def parse_chapter_content(self):
        content = self.provider.json.loads(self.provider.http_get(self.chapter_url()))
        if content['code'] != 200:
            self._error(content)
            return []
        _content = self._content(content)
        if _content is None:
            self._error(content)
        return _content



from sys import stderr
from pathlib import Path

from manga_py import meta
from manga_py.crypt.viz_com import solve
from manga_py.fs import get_util_home_path, path_join, is_file, unlink, file_size
from manga_py.provider import Provider
from .helpers.std import Std


class VizCom(Provider, Std):
    cookie_file = None
    __cookies = {}
    __has_auth = False
    _continue = True
    __is_debug = False

    def get_chapter_index(self) -> str:
        # return str(self.chapter_id)
        # todo: need tests
        idx = str(self.chapter_id)
        try:
            re = self.re.compile(r'-chapter-(\d+)/')
            idx = re.search(self.chapter).group(1)
        except AttributeError:
            print('manga-py can not get the number of the chapter!\nurl: {}'.format(self.chapter), file=stderr)
            print(' Please, report this error\n {}{}\n\n'.format(
                meta.__downloader_uri__, '/issues/new?template=bug_report.md'
            ), file=stderr)
        self.__is_debug and print('Chapter idx: {}'.format(idx))
        return idx

    def get_main_content(self):
        # return self._get_content('{}/shonenjump/chapters/{}')
        with open('/tmp/viz/viz.html', 'r') as r:
            return r.read()

    def get_manga_name(self) -> str:
        return self._get_name('/chapters/([^/]+)')

    def get_chapters(self):
        chapters = self._elements('a.flex[href*="/chapter/"],a.pad-r-rg.o_chapter-container[href*="/chapter/"]')
        self.__is_debug and print('Chapters count: %d' % len(chapters))

        if self.__is_debug:
            page = Path('viz_debug')
            page.mkdir(parents=True, exist_ok=True)
            _path = str(page.joinpath('chapters.html'))
            print('Save path to %s' % _path)
            with open(_path, 'w') as w:
                w.write('\n'.join([i.get('href') for i in chapters]))

        return chapters

    def get_files(self):
        self.__is_debug and print('Files')
        self._continue = True
        ch = self.chapter
        params = [
            'device_id=3',
            'manga_id={}'.format(self.re.search(r'/chapter/(\d+)', ch).group(1)),
            'page={page}',
        ]
        url = 'https://www.viz.com/manga/get_manga_url?' + '&'.join(params)
        self.__is_debug and print('Chapter url: %s' % url)
        if self.__has_auth:
            params.append('client_login=true')
            self.__is_debug and print('Logged client!')
        else:
            self.__is_debug and print('Anon client!')

        return [url.format(page=i) for i in range(250)]  # fixme: max 250 images per chapter

    def get_cover(self):
        self._cover_from_content('.o_hero-media')

    def prepare_cookies(self):
        self.__is_debug = self._params.get('debug', False)
        self.__is_debug = self._params
        self.http().mute = True
        self.cookie_file = path_join(get_util_home_path(), 'cookies_viz_com.dat')
        cookies = self.load_cookies()
        content = self.http().requests(self.get_url(), cookies=cookies)
        cookies.update(content.cookies.get_dict())
        self.__cookies = cookies

        if not self.has_auth():
            self.auth()
            if not self.has_auth():
                print('Warning! Login/password incorrect?\nTry to get free chapters...', file=stderr)
                print('Warning! This site worked from USA and Japan! Check your location', file=stderr)
                unlink(self.cookie_file)
                return

        self.save_cookies(self.__cookies)
        self.http().cookies = self.__cookies

    def auth(self):
        token = self.get_token()

        name = self.quest([], 'Request login on viz.com')
        password = self.quest_password('Request password on viz.com\n')

        if len(name) == 0 or len(password) == 0:
            return

        req = self.http().requests('https://www.viz.com/account/try_login', method='post', cookies=self.__cookies, data={
            'login': name,
            'pass': password,
            'rem_user': 1,
            'authenticity_token': token,
        })

        if req.status_code >= 400:
            print('Login/password error')
            exit(1)

        self.__cookies = req.cookies.get_dict()

        try:
            remember = self.json.loads(req.text)
            self.__cookies['remember_token'] = remember['trust_user_id_token_web']
        except ValueError:
            self.__is_debug and print('Remember error!', file=stderr)
            self.__is_debug and print('Please, report this error {}{}'.format(
                meta.__downloader_uri__, '/issues/new?template=bug_report.md'
            ), file=stderr)

    def save_cookies(self, cookies: dict):
        with open(self.cookie_file, 'w') as w:
            w.write(self.json.dumps(cookies))

    def load_cookies(self):
        if is_file(self.cookie_file):
            try:
                with open(self.cookie_file, 'r') as r:
                    return self.json.loads(r.read())
            except ValueError:
                unlink(self.cookie_file)
        return {}

    def get_token(self):
        auth_token_url = 'https://www.viz.com/account/refresh_login_links'
        auth_token = self.http().get(auth_token_url, cookies=self.__cookies)
        token = self.re.search(r'AUTH_TOKEN\s*=\s*"(.+?)"', auth_token)
        return token.group(1)

    def has_auth(self):
        content = self.http_get('https://www.viz.com/account/refresh_login_links', cookies=self.__cookies)
        parser = self.document_fromstring(content)
        profile = parser.cssselect('.o_profile-link')
        success = len(profile) > 0
        self.__has_auth = success
        if success:
            # print('Login as {}'.format(profile[0].text))
            print('Login success')
        return success

    @staticmethod
    def has_chapters(parser):
        return len(parser.cssselect('.o_chapter-container')) > 0

    def save_file(self, idx=None, callback=None, url=None, in_arc_name=None):
        if not self._continue:
            return

        self.__is_debug and print('\nSave file: {}'.format(idx))
        self.__is_debug and print('File url: {}'.format(url))

        _path, idx, _url = self._save_file_params_helper(url, idx)

        self.__is_debug and print('File params:\n PATH: {}\n IDX: {}\n URL: {}'.format(_path, idx, _url))

        __url = self.http_get(self.http().normalize_uri(url)).strip()

        if self.__is_debug and int(idx) < 2:
            ch = 'chapter_{}_page_{}.txt'.format(self.get_chapter_index(), idx)
            page = Path('viz_debug')
            page.mkdir(parents=True, exist_ok=True)
            __debug_path = str(page.joinpath(ch))
            print('Save path to %s' % __debug_path)
            with open(__debug_path, 'w') as w:
                print(__url)
                w.write(str(__url))
                w.close()

        if __url.find('http') != 0:
            self.__is_debug and print('\nURL is wrong: \n {}\n'.format(__url), file=stderr)
            return

        self.http().download_file(__url, _path, idx)

        if file_size(_path) < 32:
            self.__is_debug and print('File not found. Stop for this chapter')
            self._continue = False
            is_file(_path) and unlink(_path)
            return

        self.after_file_save(_path, idx)

        ref = solve(_path)
        if ref is not None:
            solved_path = _path + '-solved.jpeg'
            ref.save(solved_path)
            self._archive.add_file(solved_path, 'solved{}.jpeg'.format(idx))
            callable(callback) and callback()


main = VizCom



from manga_py.provider import Provider
from .helpers.std import Std


class ReadComicBooksOnlineOrg(Provider, Std):

    def get_chapter_index(self) -> str:
        idx = self.re.search(r'/manga/[^/]+/[^/]+[-_](\d+(?:\.\d+)?)', self.chapter)
        return idx.group(1).replace('.', '-')

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'/manga/([^/]+)')

    def get_chapters(self):
        return self._elements('#mangachapterlist .chapter > a')

    def _get_image(self, parser):
        src = parser.cssselect('a > img.mangapic')
        if not src:
            return None
        return '{}/reader/{}'.format(self.domain, src[0].get('src'))

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, 'img.mangapic')

    def get_cover(self):
        self._cover_from_content(self.content, '.field-item > a > img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ReadComicBooksOnlineOrg



from sys import stderr

import cloudscraper


class CloudFlareProtect:
    protector = []

    def run(self, url):  # pragma: no cover

        if not self.protector:
            scraper = cloudscraper.create_scraper()
            try:
                self.protector = scraper.get_tokens(url)
            except Exception as e:
                print('CF error! %s' % e, file=stderr)

        return self.protector



from os import path
from zipfile import ZipFile, ZIP_DEFLATED

from manga_py.fs import is_file, make_dirs, basename, dirname, unlink, get_temp_path
# from PIL import Image as PilImage
from manga_py.image import Image


class Archive:
    _archive = None
    _writes = None
    files = None
    not_change_files_extension = False
    no_webp = False
    has_error = False

    def __init__(self):
        self.files = []
        self._writes = {}

    def write_file(self, data, in_arc_name):
        self._writes[in_arc_name] = data

    def add_file(self, file, in_arc_name=None):
        if in_arc_name is None:
            in_arc_name = basename(file)
        self.files.append((file, in_arc_name))

    def set_files_list(self, files):
        self.files = files

    def add_book_info(self, data):
        self.write_file('comicbook.xml', data)

    def __add_files(self):
        for file in self.files:
            if is_file(file[0]):
                ext = self.__update_image_extension(file[0])
                if self.no_webp and ext[ext.rfind('.'):] == '.webp':
                    jpeg = ext[:ext.rfind('.')] + '.jpeg'
                    jpeg_path = path.join(dirname(file[0]), jpeg)
                    Image(file[0]).convert(jpeg_path)
                    file = jpeg_path, jpeg
                elif ext:
                    file = file[0], ext
                self._archive.write(*file)

    def __add_writes(self):
        for file in self._writes:
            self._archive.writestr(file, self._writes[file])

    def add_info(self, data):
        self.write_file(data, 'info.txt')

    def make(self, dst):
        if not len(self.files) and not len(self._writes):
            return

        make_dirs(dirname(dst))

        self._archive = ZipFile(dst, 'w', ZIP_DEFLATED)
        try:
            self.__add_files()
            self.__add_writes()
            self._archive.close()
        except OSError as e:
            self._archive.close()
            raise e
        self._archive.close()
        self._maked()

    def _maked(self):
        for file in self.files:
            unlink(file[0])

    def __update_image_extension(self, filename) -> str:
        fn, extension = path.splitext(filename)
        if not self.not_change_files_extension:
            ext = Image.real_extension(get_temp_path(filename))
            if ext:
                extension = ext
        return basename(fn + extension)



from sys import exit
from time import sleep

from manga_py.fs import get_util_home_path, path_join, is_file, unlink
from .e_hentai_org import EHentaiOrg


from lxml.html import HtmlElement


class ExHentaiOrg(EHentaiOrg):
    __uri = 'https://forums.e-hentai.org/index.php?act=Login&CODE={}'
    cookie_file = None

    def prepare_cookies(self):
        super().prepare_cookies()

        self.cookie_file = path_join(get_util_home_path(), 'cookies_exhentai.dat')
        if is_file(self.cookie_file):
            with open(self.cookie_file, 'r') as r:
                self._storage['cookies'] = self.json.loads(r.read())
                self.http().cookies = self._storage['cookies'].copy()
        else:
            action, method, form_data = self.prepare_form()
            content = self.http().requests(action, data=form_data, method=method.lower())
            if not ~content.text.find('You are now logged in as:'):
                print('Wrong password?')
                sleep(.1)
                exit()
            else:
                with open(self.cookie_file, 'w') as w:
                    w.write(self.json.dumps(self._storage['cookies']))

            sleep(5)

        if not self.check_panda():
            print('Panda detected. Please, try again')
            exit(1)

    def prepare_form(self):
        # Login on e-hentai!
        name = self.quest([], 'Request login on e-hentai.org')
        password = self.quest_password('Request password on e-hentai.org\n')

        selectors = [
            'input[type="hidden"]',
            'input[checked]',
            'input[type="submit"]',
        ]

        form_data = {
            'UserName': name,
            'PassWord': password,
        }
        prepare = self.http_get(self.__uri.format('00'))
        parser = self.document_fromstring(prepare, 'form[name="LOGIN"]')  # type: HtmlElement
        action = parser.get('action', self.__uri.format('01'))
        method = parser.get('method', 'get')
        for i in parser.cssselect(','.join(selectors)):  # type: HtmlElement
            form_data[i.get('name')] = i.get('value')

        return action, method, form_data

    def check_panda(self):
        success = True
        req = self.http().requests('http://exhentai.org/', method='head')
        if ~req.headers['Content-Type'].find('image/'):
            """
            if authorization was not successful
            """
            print('Sad panda detected')
            print('Cookies:\n')
            print(self.http().cookies, '\n')
            self.http().cookies = {}
            unlink(self.cookie_file)
            success = False
        req.close()

        return success


main = ExHentaiOrg



from manga_py.provider import Provider
from .helpers.std import Std


class ComicPunchNetManga(Provider, Std):
    def get_chapter_index(self) -> str:
        re = self.re.compile(r'/chapter_(\d+(?:\.\d+)?)')
        return re.search(self.chapter).group(1).replace('.', '-')

    def get_main_content(self):
        return self.http_get(self.get_url())

    def get_manga_name(self) -> str:
        return self.text_content(self.content, '.page-title')

    def get_chapters(self):
        return self._elements('.manga_chapter a')

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, 'img.picture')

    def get_cover(self) -> str:
        return self._cover_from_content('.field-name-field-pic img')


main = ComicPunchNetManga



from argparse import ArgumentParser

from loguru import logger

from .info import Info
from .providers import get_provider


class Parser:
    params = None
    provider = None

    def __init__(self, args):
        self.params = {}
        self.args = args
        self._add_params(args)

    def _add_params(self, params: ArgumentParser = None):
        if params is None:
            params = self.args.parse_args()
        else:
            params = params.parse_args()
        self.params = params.__dict__

    def init_provider(
            self,
            progress: callable = None,
            log: callable = None,
            quest: callable = None,
            info: Info = None,
            quest_password: callable = None,
    ):
        provider = get_provider(self.params.get('url', ''))
        if isinstance(provider, bool):
            raise AttributeError('Provider not found')
        self.provider = provider(info)  # provider __init__

        self.provider.set_progress_callback(progress)
        self.provider.set_log_callback(log)
        self.provider.set_quest_callback(quest)
        self.provider.set_quest_password_callback(quest_password)

    @logger.catch
    def start(self):
        self.provider.process(self.params['url'], self.params)



from manga_py.provider import Provider
from .helpers.std import Std


class MangaOnlineHereCom(Provider, Std):
    __local_storage = None

    def get_chapter_index(self) -> str:
        selector = r'/read-online/[^/]+?(\d+)(?:.(\d+))?'
        idx = self.re.search(selector, self.chapter)
        return '-'.join([
            idx[0],
            0 if idx[1] is None else idx[1]
        ])

    def get_main_content(self):
        return self._get_content('{}/manga-info/{}')

    def get_manga_name(self) -> str:
        if not self.__local_storage.get('name', None):
            url = self.get_url()
            if self.re.search(r'/read-online/', url):
                url = self.html_fromstring(url, '.back-info a', 0).get('href')
            name = self.re.search('/manga-info/Fuuka', url).group(1)
            self.__local_storage['name'] = name
        return self.__local_storage['name']

    def get_chapters(self):
        return self._elements('.list-chapter a')

    def prepare_cookies(self):
        self.__local_storage = {}

    def get_files(self):
        items = self.html_fromstring(self.chapter, '#list-img img')
        return [i.get('src') for i in items]

    def get_cover(self):
        return self._cover_from_content('.image-info img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaOnlineHereCom



import re
import importlib

providers_list = {
    '3asq_info': [
        r'3asq\.info/.',
    ],
    '7sama_com': [
        r'7sama\.com/manga/.',
    ],
    # '8muses_com': [
    #     r'8muses\.com/comics/album/.',
    # ],
    'ac_qq_com': [
        r'ac\.qq\.com/Comic.+?/id/\d',
    ],
    'acomics_ru': [
        r'acomics\.ru/~.',
    ],
    'adulto_seinagi_org': [
        r'adulto\.seinagi\.org/(series|read)/.',
        r'xanime-seduccion\.com/(series|read)/.',
        r'twistedhelscans\.com/(series|read)/.',
        r'reader\.evilflowers\.com/(series|read)/.',
    ],
    'allhentai_ru': [
        r'allhentai\.ru/.',
    ],
    'animextremist_com': [
        r'animextremist\.com/mangas-online/.',
    ],
    'antisensescans_com': [
        r'antisensescans\.com/online/(series|read)/.',
    ],
    'asmhentai_com': [
        r'asmhentai\.com/(g|gallery)/\d',
    ],
    'atfbooru_ninja': [
        r'atfbooru\.ninja/posts.',
    ],
    'authrone_com': [
        r'authrone\.com/manga/.',
    ],
    'bato_to': [
        r'bato\.to/(series|chapter)/\d',
    ],
    'blogtruyen_com': [
        r'blogtruyen\.com/.',
    ],
    'bns_shounen_ai_net': [
        r'bns\.shounen-ai\.net/read/(series|read)/.',
    ],
    'boredomsociety_xyz': [
        r'boredomsociety\.xyz/(titles/info|reader)/\d',
    ],
    'cdmnet_com_br': [
        r'cdmnet\.com\.br/titulos/.',
    ],
    'chochox_com': [
        r'chochox\.com/.',
    ],
    'choutensei_260mb_net': [
        r'choutensei\.260mb\.net/(series|read)/.',
    ],
    'comicextra_com': [
        r'comicextra\.com/.',
    ],
    # 'comico_co_id_content': [
    #     r'comico\.co\.id/content\?contentId=\d',
    # ],
    'comico_co_id_titles': [
        r'comico\.co\.id/titles/\d',
    ],
    'comic_webnewtype_com': [
        r'comic\.webnewtype\.com/contents/.',
    ],
    'comico_jp': [
        r'comico\.jp(?:/challenge)?/(detail|articleList).+titleNo.',
    ],
    'comicsandmanga_ru': [
        r'comicsandmanga\.ru/online-reading/.',
    ],
    'comicvn_net': [
        r'comicvn\.net/truyen-tranh-online/.',
    ],
    'cycomi_com': [
        r'cycomi\.com/fw/cycomibrowser/chapter/title/\d',
    ],
    'danbooru_donmai_us': [
        r'danbooru\.donmai\.us/posts.',
    ],
    'darkskyprojects_org': [
        r'darkskyprojects\.org/biblioteca/.',
    ],
    'dejameprobar_es': [
        r'dejameprobar\.es/slide/.',
        r'menudo-fansub\.com/slide/.',
        r'npscan\.mangaea\.net/slide/.',
        r'snf\.mangaea\.net/slide/.',
        r'yuri-ism\.net/slide/.',
    ],
    'desu_me': [
        r'desu\.me/manga/.',
    ],
    'digitalteam1_altervista_org': [
        r'digitalteam1\.altervista\.org/reader/read/.',
    ],
    # 'dm5_com': [
    #     r'dm5\.com/manhua-.',
    # ],
    'doujins_com': [
        r'doujins\.com/gallery/.',
        r'doujin-moe\.us/gallery/.',
    ],
    'e_hentai_org': [
        r'e-hentai\.org/g/\d',
    ],
    'exhentai_org': [
        r'exhentai\.org/g/\d',
    ],
    'fanfox_net': [
        r'fanfox\.net/manga/.',
    ],
    'freeadultcomix_com': [
        r'freeadultcomix\.com/.',
    ],
    'freemanga_to': [
        r'freemanga\.to/(manga|chapter)/.',
    ],
    'funmanga_com': [
        r'funmanga\.com/.',
    ],
    'gmanga_me': [
        r'gmanga\.me/mangas/.',
    ],
    'gomanga_co': [
        r'gomanga\.co/reader/.',
        r'jaiminisbox\.com/reader/.',
        r'kobato\.hologfx\.com/reader/.',
        # r'atelierdunoir\.org/reader/.',
        r'seinagi\.org/reader/.',
    ],
    'goodmanga_net': [
        r'goodmanga\.net/.',
    ],
    'helveticascans_com': [
        r'helveticascans\.com/r/(series|read)/.',
    ],
    'hakihome_com': [
        r'hakihome\.com/.',
    ],
    'hatigarmscans_eu': [
        r'hatigarmscans\.eu/hs/(series|read).',
        r'hatigarmscans\.net/hs/(series|read).',
        r'hatigarmscans\.net/manga/.',
    ],
    'heavenmanga_biz': [
        r'heavenmanga\.\w+/.',
    ],
    'hentai2read_com': [
        r'hentai2read\.com/.',
    ],
    'hentai_cafe': [
        r'hentai\.cafe/.',
    ],
    'hentai_chan_me': [
        r'hentai-chan\.me/(related|manga|online)/.',  # todo
    ],
    'hentai_image_com': [
        r'hentai-image\.com/image/.',
    ],
    'hentaihand_com': [
        r'hentaihand\.com/comic/\d',
    ],
    'hentaifox_com': [
        r'hentaifox\.com/.',
    ],
    'hentaihere_com': [
        r'hentaihere\.com/m/.',
    ],
    'hentaiporns_net': [
        r'hentaiporns\.net/.'
    ],
    'hentairead_com': [
        r'hentairead\.com/.',
    ],
    'hitomi_la': [
        r'hitomi\.la/(galleries|reader)/.',
    ],
    'hgamecg_com': [
        r'hgamecg\.com/index/category/\d',
    ],
    'hitmanga_eu': [
        r'hitmanga\.eu/.',
        r'mymanga\.io/.',
    ],
    'hocvientruyentranh_com': [
        r'hocvientruyentranh\.com/(manga|chapter)/.',
    ],
    'hoducomics_com': [
        r'hoducomics\.com/webtoon/list/\d',
        r'hodu1\.com/webtoon/list/\d',
    ],
    'hotchocolatescans_com': [
        r'hotchocolatescans\.com/fs/(series|read)/.',
        r'mangaichiscans\.mokkori\.fr/fs/(series|read)/.',
        r'taptaptaptaptap\.net/fs/(series|read)/.',
    ],
    'riceballicious_info': [
        r'riceballicious\.info/fs/reader/(series|read)/.',
    ],
    'rocaca_com': [
        r'rocaca\.com/manga/.',
    ],
    'inmanga_com': [
        r'inmanga\.com/ver/manga/.',
    ],
    'isekaiscan_com': [
        r'isekaiscan\.com/manga/.',
    ],
    'japscan_com': [
        r'japscan\.to/.',
    ],
    'jurnalu_ru': [
        r'jurnalu\.ru/online-reading/.',
    ],
    'kissmanga_com': [
        r'kissmanga\.com/Manga/.',
    ],
    'komikcast_com': [
        r'komikcast\.com/.',
    ],
    'komikid_com': [
        r'komikid\.com/manga/.',
        r'mangazuki\.co/manga/.',
        r'mangaforest\.com/manga/.',
        r'mangadenizi\.com/.',
        r'mangadoor\.com/manga/.',
        r'manga\.fascans\.com/manga/.',
        r'mangadesu\.net/manga/.',
        r'mangahis\.com/manga/.',
        r'cmreader\.info/manga/.',
        r'rawmangaupdate\.com/manga/.',
        r'mangaraw\.online/manga/.',
        r'manhua-tr\.com/manga/.',
        r'manga-v2\.mangavadisi\.org/manga/.',
        r'universoyuri\.com/manga/.',
        r'digitalteam1\.altervista\.org/manga/.',
        r'sosscanlation\.com/manga/.',
        r'komikgue\.com/manga/.',
    ],
    'kumanga_com': [
        r'kumanga\.com/manga/\d',
    ],
    'lector_kirishimafansub_com': [
        r'lector\.kirishimafansub\.com/(lector/)?(series|read)/.',
    ],
    'leitor_net': [
        r'leitor\.net/manga/.',
    ],
    'leomanga_com': [
        r'leomanga\.com/manga/.',
    ],
    'leviatanscans_com': [
        r'leviatanscans\.com/comics/\d'
    ],
    'lhtranslation_com': [
        r'read\.lhtranslation\.com/(truyen|manga)-.',
        r'lhtranslation\.net/(truyen|manga)-.',
    ],
    'lolibooru_moe': [
        r'lolibooru\.moe/post.',
    ],
    'lolivault_net': [
        r'lolivault\.net/online/(series|read).',
    ],
    'luscious_net': [
        r'luscious\.net/.+/album/.',
        r'luscious\.net/albums/.',
    ],
    'mangapark_org': [
        r'mangapark\.org/(series|chapter)/',  # is different!
    ],
    'mang_as': [
        r'mang\.as/manga/.',
    ],
    'manga_ae': [
        r'manga\.ae/.',
    ],
    'manga_fox_com': [
        r'manga-fox\.com/.',
        r'manga-here\.io/.',
    ],
    'manga_mexat_com': [
        r'manga\.mexat\.com/category/.',
    ],
    'manga_online_biz': [
        r'manga-online\.biz/.',
    ],
    'manga_online_com_ua': [
        r'manga-online\.com\.ua/.+html',
    ],
    'manga_room_com': [
        r'manga-room\.com/manga/.',
    ],
    'manga_sh': [
        r'manga\.sh/comics/.',
    ],
    'manga_tube_me': [
        r'manga-tube\.me/series/.',
    ],
    'manga_tr_com': [
        r'manga-tr\.com/(manga|id)-.',
    ],
    'mangabat_com': [
        r'mangabat\.com/(manga|chapter)/.',
    ],
    'mangabb_co': [
        r'mangabb\.co/.',
    ],
    'mangabox_me': [
        r'mangabox\.me/reader/.',
    ],
    'mangachan_me': [
        r'mangachan\.me/(related|manga|online)/.',
        r'yaoichan\.me/(manga|online).',
    ],
    'mangachan_me_download': [
        r'mangachan\.me/download/.',
        r'hentai-chan\.me/download/.',
        r'yaoichan\.me/download/.',
    ],
    'mangacanblog_com': [
        r'mangacanblog\.com/.',
    ],
    'mangaclub_ru': [
        r'mangaclub\.ru/.',
    ],
    'mangadeep_com': [
        r'mangadeep\.com/.',
        r'manga99\.com/.',
    ],
    'mangadex_com': [
        r'mangadex\.com/(title|chapter)/.',
        r'mangadex\.org/(title|chapter)/.',
    ],
    'mangadex_info': [
        r'mangadex\.info/manga/.',
    ],
    'mangaeden_com': [
        r'mangaeden\.com/[^/]+/[^/]+-manga/.',
        r'perveden\.com/[^/]+/[^/]+-manga/.',
    ],
    'mangafans_us': [  # MangaNeloCom
        r'mangafans\.us/manga/.',
        r'mangahot\.org/read-manga/.',
    ],
    'mangaforall_com': [
        r'mangaforall\.com/m/.',
    ],
    'mangafreak_net_download': [
        r'mangafreak\.net/Manga/.',
    ],
    'mangafull_org': [
        r'mangafull\.org/manga/.',
    ],
    # 'mangago_me': [
    #     r'mangago\.me/read-manga/.',
    # ],
    'mangahasu_se': [
        r'mangahasu\.se/.',
    ],
    'mangaheaven_club': [
        r'mangaheaven\.club/read-manga/.',
    ],
    'mangaheaven_xyz': [
        r'mangaheaven\.xyz/manga/.',
    ],
    'mangahere_cc': [
        r'mangahere\.co/manga/.',
        r'mangahere\.cc/manga/.',
    ],
    'mangahi_net': [
        r'mangahi\.net/.',
    ],
    'mangaid_me': [
        r'mangaid\.co/manga/.',
        r'mangaid\.net/manga/.',
        r'mangaid\.me/manga/.',
    ],
    'mangahome_com': [
        r'mangahome\.com/manga/.',
    ],
    'mangahub_io': [
        r'mangahub\.io/(manga|chapter)/.',
        r'mangareader\.site/(manga|chapter)/.',
        r'mangakakalot\.fun/(manga|chapter)/.',
        r'mangahere\.onl/(manga|chapter)/.',
    ],
    'mangahub_ru': [
        r'mangahub\.ru/.',
    ],
    'mangaindo_web_id': [
        r'mangaindo\.web\.id/.',
    ],
    'mangainn_net': [
        r'mangainn\.net/.',
    ],
    'mangajinnofansub_com': [  # normal
        r'mangajinnofansub\.com/lector/(series|read)/.',
    ],
    'mangakakalot_com': [
        r'mangakakalot\.com/(manga|chapter)/.',
    ],
    'mangakatana_com': [
        r'mangakatana\.com/manga/.',
    ],
    'mangaku_web_id': [
        # r'mangaku\.web\.id/.',
        r'mangaku\.in/.',
    ],
    'mangalib_me': [
        r'mangalib\.me/.',
    ],
    'mangalife_us': [
        r'mangalife\.us/(read-online|manga)/.',
    ],
    'mangamew_com': [
        r'mangamew\.com/(\w+-)?manga/.',
    ],
    'mangamew_com_vn': [
        r'mangamew\.com/(\w+-)?truyen/.',
    ],
    'manganelo_com': [
        r'manganelo\.com/(manga|chapter)/.',
    ],
    'mangaon_net': [
        r'mangaon\.net/(manga-info|read-online)/.',
    ],
    'mangaonline_com_br': [
        r'mangaonline\.com\.br/.',
    ],
    'mangaonline_today': [
        r'mangaonline\.today/.',
    ],
    'mangaonlinehere_com': [
        r'mangaonlinehere\.com/(manga-info|read-online)/.',
    ],
    'mangapanda_com': [
        r'mangapanda\.com/.',
    ],
    'mangapark_me': [
        r'mangapark\.me/manga/.',
    ],
    'mangareader_net': [
        r'mangareader\.net/.',
    ],
    'mangareader_xyz': [
        r'mangareader\.xyz/manga/.',
        r'mangareader\.xyz/.+?/chapter-\d',
        # r'mangafox\.cc/manga/.',
        # r'mangafox\.cc/.+?/chapter-\d',
    ],
    'mangarock_com': [
        r'mangarock\.com/manga/.',
    ],
    'mangarussia_com': [
        r'mangarussia\.com/(manga|chapter)/.',
    ],
    'mangasaurus_com': [
        r'mangasaurus\.com/(manga|view).',
    ],
    'mangaseeonline_us': [
        r'mangaseeonline\.us/(read-online|manga)/.',
    ],
    'mangashiro_net': [
        r'mangashiro\.net/.',
    ],
    'mangasupa_com': [
        r'mangasupa\.com/(manga|chapter)/.',
    ],
    'mangasushi_net': [
        r'mangasushi\.net/manga/.',
    ],
    'mangatail_com': [
        r'mangasail\.com/(manga|chapter|node|content)/.',
        r'mangasail\.co/(manga|chapter|node|content)/.',
        r'mangatail\.me/(manga|chapter|node|content)/.',
    ],
    'mangatown_com': [
        r'mangatown\.com/manga/.',
    ],
    'mangatrue_com': [
        r'mangatrue\.com/manga/.',
        r'mangaall\.com/manga/.',
    ],
    'mangawindow_net': [
        r'mangawindow\.net/(series|chapter)/\d',  # is different!
    ],
    'mangax_net': [
        r'mangax\.net/\w/.',
    ],
    'mangazuki_me': [
        r'mangazuki\.me/manga/.',
        r'mangazuki\.info/manga/.',
        r'mangazuki\.online/manga/.',
    ],
    'manhuagui_com': [
        r'manhuagui\.com/comic/\d',
    ],
    'manhuatai_com': [
        r'manhuatai\.com/.',
    ],
    'manhwa_co': [
        r'manhwa\.co/.',
    ],
    # 'manhwahentai_com': [
    #     r'manhwahentai\.com/manhwa/.'
    # ],
    'merakiscans_com': [
        r'merakiscans\.com/manga/.',
    ],
    'mintmanga_com': [
        r'mintmanga\.com/.',
    ],
    'mngcow_co': [
        r'mngcow\.co/.',
    ],
    'mngdoom_com': [
        r'mangadoom\.co/.',
        r'mngdoom\.com/.',
    ],
    'mymangalist_org': [
        r'mymangalist.org/(read|chapter)-',
    ],
    'myreadingmanga_info': [
        r'myreadingmanga\.info/.',
    ],
    'neumanga_tv': [
        r'neumanga\.tv/manga/.',
    ],
    'nhentai_net': [
        r'nhentai\.net/g/.',
    ],
    'niadd_com': [
        r'niadd\.com/manga/.',
    ],
    'nightow_net': [
        r'nightow\.net/online/\?manga=.',
    ],
    'nineanime_com': [
        r'nineanime\.com/manga/.+\.html'
    ],
    'ninemanga_com': [
        r'ninemanga\.com/(manga|chapter).',
        r'addfunny\.com/(manga|chapter).',
    ],
    'noranofansub_com': [
        r'noranofansub\.com(/lector)?/(series/|read/)?.',
    ],
    'nozominofansub_com': [  # mangazuki_co
        r'nozominofansub\.com/public(/index\.php)?/manga/.',
        r'godsrealmscan\.com/public(/index\.php)?/manga/.',
    ],
    'nude_moon_me': [
        r'nude-moon\.me/\d',
    ],
    'otakusmash_com': [
        r'otakusmash\.com/.',
        r'mrsmanga\.com/.',
        r'mentalmanga\.com/.',
        r'mangasmash\.com/.',
        r'omgbeaupeep\.com/comics/.',
    ],
    'otscans_com': [
        r'otscans\.com/foolslide/(series|read)/.',
    ],
    'pecintakomik_com_manga': [
        r'pecintakomik\.com/manga/.',
    ],
    'pecintakomik_com': [
        r'pecintakomik\.com/.',
    ],
    'plus_comico_jp_manga': [
        r'plus\.comico\.jp/manga/\d',
    ],
    'plus_comico_jp': [
        r'plus\.comico\.jp/store/\d',
    ],
    'porncomix_info': [
        r'porncomix\.info/.',
    ],
    'psychoplay_co': [
        r'psychoplay\.co/(series|read)/.',
    ],
    'puzzmos_com': [
        r'puzzmos\.com/manga/.',
    ],
    r'pururin_io': [
        r'pururin\.io/(gallery|read)/.',
    ],
    'pzykosis666hfansub_com': [
        r'pzykosis666hfansub\.com/online/.',
    ],
    'ravens_scans_com': [
        r'ravens-scans\.com(/lector)?/(serie/|read/).',
    ],
    'raw_senmanga_com': [
        r'raw\.senmanga\.com/.',
    ],
    'rawdevart_com': [
        r'rawdevart\.com/manga/.',
    ],
    'rawlh_com': [
        r'lhscan\.net/(truyen|manga|read)-.',
        r'rawqq\.com/(truyen|manga|read)-.',
        r'rawqv\.com/(truyen|manga|read)-.',
    ],
    'rawneko_com': [
        r'rawneko\.com/manga/.',
    ],
    'read_egscans_com': [
        r'read\.egscans\.com/.',
    ],
    'read_powermanga_org': [
        r'lector\.dangolinenofansub\.com/(series|read)/.',
        r'read\.powermanga\.org/(series|read)/.',
        # r'read\.yagami\.me/(series|read)/.',
        r'reader\.kireicake\.com/(series|read)/.',
        r'reader\.shoujosense\.com/(series|read)/.',
        r'reader\.sensescans\.com/(series|read)/.',
        r'reader\.whiteoutscans\.com/(series|read)/.',
        r'slide\.world-three\.org/(series|read)/.',
        r'manga\.animefrontline\.com/(series|read)/.',
        r'reader\.s2smanga\.com/(series|read)/.',
        r'reader\.seaotterscans\.com/(series|read)/.',
        r'reader\.idkscans\.com/(series|read)/.',
        r'reader\.thecatscans\.com/(series|read)/.',
        r'reader\.deathtollscans\.net/(series|read)/.',
        r'lector\.ytnofan\.com/(series|read)/.',
        r'reader\.jokerfansub\.com/(series|read)/.',
        r'lector\.patyscans\.com/(series|read)/.',
        r'truecolorsscans\.miocio\.org/(series|read)/.',
        r'reader\.letitgo\.scans\.today/(series|read)/.',
        r'reader\.fos-scans\.com/(series|read)/.',
        r'reader\.serenade\.moe/(series|read)/.',
        r'reader\.vortex-scans\.com/(series|read)/.',
        r'reader\.roseliascans\.com/(series|read)/.',
        r'reader\.silentsky-scans\.net/(series|read)/.',
        r'hoshiscans\.shounen-ai\.net/(series|read)/.',
        r'digitalteamreader\.netsons\.org/(series|read)/.',
        r'reader\.manga-download\.org/(series|read)/.',
    ],
    'read_yagami_me': [
        r'read\.yagami\.me/series/\w',
    ],
    # 'readcomicbooksonline_org_manga': [  # todo #168
    #     r'readcomicbooksonline\.net/manga/.',
    #     r'readcomicbooksonline\.org/manga/.',
    # ],
    # 'readcomicbooksonline_org': [
    #     r'readcomicbooksonline\.net/.',
    #     r'readcomicbooksonline\.org/.',
    # ],
    'comicpunch_net_manga': [
        r'comicpunch\.net/asiancomics/.',
    ],
    'comicpunch_net': [
        r'comicpunch\.net/.',
    ],
    'reader_championscans_com': [
        r'reader\.championscans\.com/(series|read)/.',
    ],
    'reader_imangascans_org': [
        r'reader\.imangascans\.org/.',
    ],
    'readhentaimanga_com': [
        r'readhentaimanga\.com/.',
    ],
    'readcomiconline_to': [
        r'readcomiconline\.to/Comic/.',
    ],
    'readcomicsonline_ru': [
        r'readcomicsonline\.ru/comic/.',
    ],
    'readmanga_me': [
        r'readmanga\.me/.',
    ],
    'readmanga_eu': [
        r'readmanga\.eu/manga/\d+/.',
    ],
    'readmng_com': [
        r'readmng\.com/.',
    ],
    'readms_net': [
        r'readms\.net/(r|manga)/.',
    ],
    'remanga_org': [
        r'remanga\.org/manga/.',
    ],
    'santosfansub_com': [
        r'santosfansub\.com/Slide/.',
    ],
    'selfmanga_ru': [
        r'selfmanga\.ru/.',
    ],
    'senmanga_com': [
        r'senmanga\.com/.',
    ],
    'shakai_ru': [
        r'shakai\.ru/manga.*?/\d',
    ],
    'shogakukan_co_jp': [
        r'shogakukan\.co\.jp/books/\d',
        r'shogakukan\.co\.jp/magazines/series/\d',
    ],
    'shogakukan_tameshiyo_me': [
        r'shogakukan\.tameshiyo\.me/\d',
    ],
    'siberowl_com': [
        r'siberowl\.com/mangas/.',
    ],
    'sleepypandascans_co': [
        r'sleepypandascans\.co/(Series|Reader)/.',
    ],
    'somanga_net': [
        r'somanga\.net/(leitor|manga)/.',
        r'somangas\.net/(leitor|manga)/.',
    ],
    'subapics_com': [
        r'subapics\.com/manga/.',
        r'subapics\.com/.+-chapter-.',
        r'mangakita\.net/manga/.',
        r'mangakita\.net/.+-chapter-.',
        r'komikstation\.com/manga/.',
        r'komikstation\.com/.+-chapter-.',
        r'mangavy\.com/manga/.',
        r'mangavy\.com/.+-chapter-.',
        r'mangakid\.net/manga/.',
        r'mangakid\.net/.+-chapter-.',
    ],
    'submanga_online': [
        r'submanga\.online/manga/.',
    ],
    'sunday_webry_com': [
        r'sunday-webry\.com/series/\d',
    ],
    'taadd_com': [
        r'taadd\.com/(book|chapter)/.',
    ],
    'tapas_io': [
        r'tapas\.io/episode/\d',
        r'tapas\.io/series/\w',
    ],
    'tenmanga_com': [
        r'tenmanga\.com/(book|chapter)/.',
    ],
    'tmofans_com': [
        r'tmofans\.com/library/manga/\d',
    ],
    'translate_webtoons_com': [
        r'translate\.webtoons\.com/webtoonVersion\?webtoonNo.',
    ],
    'trashscanlations_com': [
        r'trashscanlations\.com/series/.',
    ],
    'tonarinoyj_jp': [
        r'tonarinoyj\.jp/episode/.',
    ],
    'toonkor_co': [
        r'toonkor\.co/.',
    ],
    'triplesevenscans_com': [
        r'triplesevenscans\.com/reader/(series|read)/.',
        r'cm-scans\.shounen-ai\.net/reader/(series|read)/.',
        r'yaoislife\.shounen-ai\.net/reader/(series|read)/.',
        r'fujoshibitches\.shounen-ai\.net/reader/(series|read)/.',
    ],
    'truyen_vnsharing_site': [
        r'truyen\.vnsharing\.site/index/read/.',
    ],
    'truyenchon_com': [
        r'truyenchon\.com/truyen/.',
        r'nettruyen\.com/truyen-tranh/.',
    ],
    'truyentranhtuan_com': [
        r'truyentranhtuan\.com/.',
    ],
    'tsumino_com': [
        r'tsumino\.com/Book/Info/\d',
        r'tsumino\.com/Read/View/\d',
    ],
    # 'tumangaonline_com': [
    #     r'tumangaonline\.com/.',
    #     r'tumangaonline\.me/.',
    # ],
    'unionmangas_net': [
        r'unionmangas\.cc/(leitor|manga)/.',
        r'unionmangas\.net/(leitor|manga)/.',
        r'unionmangas\.site/(leitor|manga)/.',
    ],
    'viz_com': [
        r'viz\.com/shonenjump/chapters/.',
    ],
    'web_ace_jp': [
        r'web-ace\.jp/youngaceup/contents/\d',
    ],
    'webtoon_bamtoki_com': [
        r'webtoon\.bamtoki\.com/.',
        r'webtoon\.bamtoki\.se/.',
    ],
    'webtoons_com': [
        r'webtoons\.com/[^/]+/[^/]+/.',
    ],
    'webtoontr_com': [
        r'webtoontr\.com/_/.',
    ],
    'westmanga_info': [
        r'westmanga\.info/.',
    ],
    'whitecloudpavilion_com': [
        r'whitecloudpavilion\.com/manga/free/manga/.',
    ],
    'wiemanga_com': [
        r'wiemanga\.com/(manga|chapter)/.',
    ],
    'wmanga_ru': [
        r'wmanga\.ru/starter/manga_.',
    ],
    'yande_re': [
        r'yande\.re/post.',
    ],
    'zeroscans_com': [
        r'zeroscans\.com/manga/.',
        r'manhwareader\.com/manga/.',
    ],
    'zingbox_me': [
        r'zingbox\.me/.',
    ],
    'zip_read_com': [
        r'zip-read\.com/.',
    ],
    'zmanga_net': [
        r'zmanga\.net/.',
    ],
}


def __check_provider(provider, url):
    items = [r'\b' + i for i in provider]
    reg = '(?:' + '|'.join(items) + ')'
    return re.search(reg, url)


def get_provider(url):
    fromlist = 'manga_py.providers'
    for i in providers_list:
        if __check_provider(providers_list[i], url):
            provider = importlib.import_module('%s.%s' % (fromlist, i))
            return provider.main
    return False



from manga_py.provider import Provider
from .helpers.std import Std


class MangaRoomCom(Provider, Std):

    def get_archive_name(self) -> str:  # fixme! #49
        idx = self.get_chapter_index().split('-')
        if len(idx) > 1:
            return '{}vol_{:0>3}'.format(*idx)
        return 'vol_{:0>3}'.format(idx[0])

    def get_chapter_index(self) -> str:
        re = self.re.search(r'version-(\d+)', self.chapter)
        version = ''
        if re:
            version = '-' + re.group(1)
        re = self.re.search(r'chapter-(\d+)', self.chapter)
        return '{}{}'.format(version, re.group(1))

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)')

    def get_chapters(self):
        # : downloaded all versions
        return ['{}/manga/{}'.format(
            self.domain, i.get('href')
        ) for i in self._elements('.chapter-list a')]

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '#main_div .lazy_image_page')

    def get_cover(self) -> str:
        return self._cover_from_content('img#manga_thumps')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaRoomCom



from .rawdevart_com import RawDevArtCom


class ISekaiScanCom(RawDevArtCom):
    _chapter_selector = r'/chapter-(\d+(?:-[\w\-]+)?)'

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '.page-break img.wp-manga-chapter-img', 'data-src')


main = ISekaiScanCom



from .authrone_com import AuthroneCom
from .helpers.std import Std


class MngCowCo(AuthroneCom, Std):

    def get_chapter_index(self) -> str:
        return self.re.search(
            r'\.co/[^/]+/([^/]+)',
            self.chapter
        ).group(1).replace('.', '-')

    def get_main_content(self):
        return self._get_content('{}/{}/')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.co/([^/]+)')


main = MngCowCo



from manga_py.provider import Provider
from .helpers.std import Std


class MangaParkMe(Provider, Std):

    def get_chapter_index(self) -> str:
        selector = r'/manga/[^/]+/s.+?(?:/v(\d+))?/c(\d+[^/]*)'
        idx = self.re.search(selector, self.chapter).groups()
        if idx[0] is None:
            return '0-' + idx[1]
        return '-'.join(idx)

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)')

    def get_chapters(self):
        return self._elements('#list a.ch')

    def get_files(self):
        content = self.http_get(self.chapter)
        data = self.re.search(r'var\simages\s?=\s?(\[.+\])', content)
        if not data:
            return []
        return self.json.loads(data.group(1))

    def get_cover(self):
        return self._cover_from_content('.cover img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaParkMe



#!/usr/bin/python3
# -*- coding: utf-8 -*-

from __future__ import print_function

from os import path

from setuptools import setup

from manga_py import __author__, __email__, __license__
from manga_py.meta import __version__, __downloader_uri__

REQUIREMENTS = [
    'lxml',
    'cssselect',
    'Pillow',
    'requests',
    'pycrypto',
    'cloudscraper',
    'progressbar2',
    'urllib3',
    'packaging',
    'pyexecjs',
    'html-purifier',
    'selenium',
    'loguru',
]


# if path.isfile('requirements.txt'):
#     with open('requirements.txt') as f:
#         REQUIREMENTS = f.read()


long_description = ''
if path.isfile('README.rst'):
    with open('README.rst') as f:
        long_description = f.read()


release_status = 'Development Status :: 5 - Production/Stable'
if ~__version__.find('beta'):
    release_status = 'Development Status :: 4 - Beta'
if ~__version__.find('alpha'):
    release_status = 'Development Status :: 3 - Alpha'


setup(
    name='manga_py',
    packages=[
        'manga_py',
        'manga_py.base_classes',
        'manga_py.crypt',
        'manga_py.cli',
        'manga_py.http',
        'manga_py.providers',
        'manga_py.providers.helpers',
        'manga_py.server',
    ],
    include_package_data=True,
    version=__version__,
    description='Universal assistant download manga.',
    long_description=long_description,
    author=__author__,
    author_email=__email__,
    url=__downloader_uri__,
    zip_safe=False,
    data_files=[
        ('manga_py/storage', [
            'manga_py/storage/.passwords.json.dist',
            'manga_py/storage/.proxy.txt',
            'manga_py/crypt/aes.js',
            'manga_py/crypt/aes_zp.js',
        ]),
    ],
    download_url='{}/archive/{}.tar.gz'.format(__downloader_uri__, __version__),
    keywords=['manga-downloader', 'manga', 'manga-py'],
    license=__license__,
    classifiers=[  # look here https://pypi.python.org/pypi?%3Aaction=list_classifiers
        release_status,
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Environment :: Console',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP',
    ],
    python_requires='>=3.5',
    install_requires=REQUIREMENTS,
    entry_points={
        'console_scripts': [
            'manga-py = manga_py:main',
        ]
    }
)



from manga_py.provider import Provider
from .helpers.std import Std


class MangaEdenCom(Provider, Std):
    uriRegex = r'/([^/]+)/[^/]+-manga/([^/]+)/?'
    apiUri = '{}/api/{}/{}/'  # (domain, chapter|manga,
    __lang = 'en'
    __cdn_url = 'https://cdn.mangaeden.com/mangasimg/'

    def get_chapter_index(self) -> str:
        return str(self.chapter[0]).replace('.', '-')

    def get_main_content(self):
        return self.http_get('{domain}/{lang}/{lang}-manga/{name}/'.format(
            domain=self.domain,
            lang=self.__lang,
            name=self.manga_name,
        ))

    def get_manga_name(self) -> str:
        re = self.re.search(self.uriRegex, self.get_url())
        self.__lang = re.group(1)
        return re.group(2)

    def get_chapters(self):  # issue #61
        manga_idx = self.re.search(r'.manga_id2\s?=\s?"(.+?)";', self.content).group(1)
        return self.json.loads(self.http_get(self.apiUri.format(
            self.domain,
            'manga',
            manga_idx,
        ))).get('chapters', [])

    def get_files(self):
        items = self.json.loads(self.http_get(self.apiUri.format(
            self.domain,
            'chapter',
            self.chapter[3]
        ))).get('images', [])
        return [self.__cdn_url + i[1] for i in items]

    def get_cover(self) -> str:
        return self._cover_from_content('#rightContent .info img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaEdenCom



from requests import get
from time import sleep


class Std:
    _vol_fill = False

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index()
        self._vol_fill = True
        return self.normal_arc_name({'vol': idx.split('-')})

    def _elements(self, selector, content=None) -> list:
        if not content:
            content = self.content
        return self.document_fromstring(content, selector)

    def _cover_from_content(self, selector, attr='src') -> str:
        image = self._elements(selector)
        if image is not None and len(image):
            return self.http().normalize_uri(image[0].get(attr))

    @staticmethod
    def _first_select_options(parser, selector, skip_first=True) -> list:
        options = 'option'
        if skip_first:
            options = 'option + option'
        select = parser.cssselect(selector)
        if select:
            return select[0].cssselect(options)
        return []

    @classmethod
    def _images_helper(cls, parser, selector, attr='src') -> list:
        image = parser.cssselect(selector)
        return [i.get(attr).strip(' \r\n\t\0') for i in image]

    @classmethod
    def _idx_to_x2(cls, idx, default=0) -> list:
        return [
            str(idx[0]),
            str(default if len(idx) < 2 or not idx[1] else idx[1])
        ]

    @staticmethod
    def _join_groups(idx, glue='-') -> str:
        result = []
        for i in idx:
            if i:
                result.append(i)
        return glue.join(result)

    def _get_name(self, selector, url=None) -> str:
        if url is None:
            url = self.get_url()
        return self.re.search(selector, url).group(1)

    def _get_content(self, tpl) -> str:
        return self.http_get(tpl.format(self.domain, self.manga_name))

    def _base_cookies(self, url=None):
        if url is None:
            url = self.get_url()
        cookies = self.http().get_base_cookies(url)
        self._storage['cookies'] = cookies.get_dict()

    def parse_background(self, image) -> str:
        selector = r'background.+?url\([\'"]?([^\s]+?)[\'"]?\)'
        url = self.re.search(selector, image.get('style'))
        return self.http().normalize_uri(url.group(1))

    @property
    def manga_name(self) -> str:
        name = self._storage.get('manga_name', None)
        if name is None:
            name = self.get_manga_name()
        return name

    def normal_arc_name(self, idx):
        if isinstance(idx, str):
            idx = [idx]
        if isinstance(idx, list):
            self._vol_fill = True
            return self.__normal_name_list(idx)
        if isinstance(idx, dict):
            return self.__normal_name_dict(idx)
        raise DeprecationWarning('Wrong arc name type: %s' % type(idx))

    @staticmethod
    def __fill(var, fmt: str = '-{}'):
        if isinstance(var, str):
            var = [var]
        return (fmt * len(var)).format(*var).lstrip('-')

    def __normal_name_list(self, idx: list):
        fmt = 'vol_{:0>3}'
        if len(idx) > 1:
            fmt += '-{}' * (len(idx) - 1)
        elif self._vol_fill and self._zero_fill:
            idx.append('0')
            fmt += '-{}'
        return fmt.format(*idx)

    def __normal_name_dict(self, idx: dict):
        vol = idx.get('vol', None)
        ch = idx.get('ch', None)
        result = ''
        if vol:
            if isinstance(vol, str):
                vol = [vol]
            result = self.__normal_name_list(vol)
        if ch:
            result += '-ch_' + self.__fill(ch)
        if self._with_manga_name:
            result = '%s-%s' % (self.manga_name, result)
        return result

    def text_content(self, content, selector, idx: int = 0, strip: bool = True):
        doc = self.document_fromstring(content, selector)
        if not doc:
            return None
        text = doc[idx].text_content()
        if strip:
            text = text.strip()
        return text

    def _download(self, file_name, url, method):
        # clean file downloader
        now_try_count = 0
        while now_try_count < 5:
            with open(file_name, 'wb') as out_file:
                now_try_count += 1
                response = get(url, timeout=60, allow_redirects=True)
                if response.status_code >= 400:
                    self.http().debug and print('ERROR! Code {}\nUrl: {}'.format(
                        response.status_code,
                        url,
                    ))
                    sleep(2)
                    continue
                out_file.write(response.content)
                response.close()
                out_file.close()
                break



from abc import ABCMeta
from time import sleep
from urllib.parse import unquote

from requests import get

from manga_py.provider import Provider


class NineHelper(Provider, metaclass=ABCMeta):
    img_server = 'https://ta1.taadd.com'

    def re_name(self, url):
        return self.re.search(r'/manga/(.+)\.html', url)

    @staticmethod
    def normalize_name(name, normalize):
        if normalize:
            name = unquote(name)
        return name

    def parse_img_uri(self, url):
        return self.re.search('://[^/]+/(.+)', url).group(1)

    def get_img_server(self, content):
        server = self.re.search(r'img_url\s?=\s?"([^"]+)', content)
        if server:
            return server.group(1)
        return self.img_server

    def get_files_on_page(self, content):
        result = self.document_fromstring(content, 'em a.pic_download')
        if not result:
            return []
        images = []
        pic_url = self.get_img_server(content)
        for i in result:
            src = self.parse_img_uri(i.get('href'))
            images.append('{}/{}'.format(pic_url, src))
        return images

    def _get_page_content(self, url):
        sleep(.6)
        return get(
            url,
            headers={
                'Accept-Language': 'ru-RU,ru;q=0.8,en-US;q=0.5,en;q=0.3',
                'Referer': '',
            }  # fix guard
        ).text

    def prepare_cookies(self):
        self._storage['cookies'].setdefault('__cfduid', '1a2b3c4d5e')



import html

from manga_py.provider import Provider
from .helpers.std import Std


class MangaHubRu(Provider, Std):

    def get_chapter_index(self) -> str:
        idx = self.re.search(r'/read/[^/]+/[^\d]+(\d+)/(\d+)/', self.chapter).groups()
        return '{}-{}'.format(*idx)

    def get_main_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.ru/([^/]+)/?')

    def get_chapters(self):
        return self._elements('.b-catalog-list__name a[href^="/"]')

    def get_files(self):
        parser = self.html_fromstring(self.chapter, '.b-main-container .b-reader__full')
        if not parser:
            return []
        result = parser[0].get('data-js-scans')
        result = self.json.loads(html.unescape(result.replace(r'\/', '/')))
        n = self.http().normalize_uri
        return [n(i['src']) for i in result]

    def get_cover(self):
        return self._cover_from_content('.manga-section-image__img img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaHubRu



from manga_py.provider import Provider
from .helpers.std import Std


class MangaDexCom(Provider, Std):
    _content = None

    def get_chapter_index(self) -> str:
        return self.re.search(r'-chapter-(\d+(?:\.\d+)?)', self.chapter).group(1).replace('.', '-')

    def get_main_content(self):
        if self._content is None:
            self._content = self.http_get(self.get_url())
        return self._content

    def get_manga_name(self) -> str:
        return self.text_content(self.content, '.info-title')

    def get_chapters(self):
        return self._elements('.widget .table.table-striped td > a')

    def get_files(self):
        parser = self.html_fromstring(self.chapter + '/0')
        return self._images_helper(parser, '#view-chapter img')

    def get_cover(self) -> str:
        return self._cover_from_content('.img-thumbnail')

    def prepare_cookies(self):
        self.http().allow_send_referer = False


main = MangaDexCom
